# Config.pm - The config file for the Portal.
# Created by James A. Pattie (james@pcxperience.com)
# 11/13/2000 - Copyright (c) 2000-2003, Xperience, Inc.
# (http://www.pcxperience.com/)
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Data::Config;

# notes: updated on 6/30/2001 to work with the modifiable data in /etc/pcx_portal.
# You must now wrap the Portal::Data::Config->new method in an eval statement to
# know if an error occurs.  You can also specify a prefix to use before /etc/pcx_portal
# so that this can be more rpm building friendly.

# updated on 07/25/2001 to enforce what version of the config file we are supposed to
# be working with.  Moved the static color definitions into the config file.

# updated on 04/18/2002 to work with XML config files.

use strict;
use Portal::Base;
use Portal::XML::ConfigParser;
use vars qw ($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter);
@EXPORT = qw();

$VERSION = '0.10';

my $configVersion = "0.6";  # This must be kept in sync with the setup.pl script.

=head1 NAME

Config - Object used to store the Portal Configuration Data.

=head1 SYNOPSIS

  use Portal::Data::Config;
  my $obj = undef;
  eval { $obj = Portal::Data::Config->new(langObj => $langObj); };
  if ($@) # error occurred!
  {
    die "Error instantiating Config object! $@";
  }
  print "myHostName = $obj->{myHostName}\n";

=head1 DESCRIPTION

Config is the Portal Data Configuration class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(prefix)

 Creates a new instance of the Portal::Data::Config module.
 See Portal::Base(3) for a listing of required arguments.

 optional: prefix - prefix directory to go before /etc/pcx_portal.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( prefix => "", @_ );
  my $errStr = "Portal::Data::Config->new$self->{errorPhrase}";

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  my $prefix = $args{prefix};

  # source in the config file from $prefix/etc/pcx_portal/Config.pm and suck the settings hash into ourselves.
  my $cfgFile = "$prefix/etc/pcx_portal/Portal.xml";

  eval { $self->{cfgObj} = Portal::XML::ConfigParser->new(langObj => $self->{langObj}); };
  if ($@)
  {
    die $errStr . "Instantiation of ConfigParser failed!<br>\n$@";
  }
  if ($self->{cfgObj}->didErrorOccur)
  {
    die $errStr . $self->{cfgObj}->errorMessage;
  }
  eval { $self->{settingsObj} = $self->{cfgObj}->parse(file => $cfgFile, module => "Portal", version => $configVersion); };
  if ($@)
  {
    die $errStr . "Parse of '$cfgFile' failed!<br>\n$@";
  }
  if ($self->{settingsObj}->didErrorOccur)
  {
    die $errStr . $self->{settingsObj}->errorMessage;
  }

  my %settings = ();
  %settings = %{$self->{settingsObj}->{settings}};

  foreach my $name (keys %settings)
  {
    $self->{$name} = $settings{$name};
  }

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  if ($self->{settingsObj}->{version} ne $configVersion)
  {
    die $errStr . "Config File '$cfgFile' is at version '$self->{settingsObj}->{version}' not at the current version of '$configVersion'!<br>\n";
  }

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 Exported VARIABLES

  version       - Version of Config File being used
  myHostName    - Portal host name or IP Address
  httpType      - Type of Web Server (http or https)
  sessionType   - Type of Sessions (Database or DB_File)
  sessionDirectory - Directory to use for Session Files
  sessionLockDir- Directory to use for File Locking
  dbType        - Database Type (Postgres or MySQL)
  dbHost        - host name or IP Address of Database Server
  dbName        - name of the Portal database
  dbPort        - Database Port to connect to
  dbAdminUser   - Admin User to connect to database as
  dbAdminPasswd - Admin Password to use when connecting to the database
  dbUser        - User to connect to database as
  dbPasswd      - Password to use when connecting to the database
  billingdbHost - host name or IP Address of Billing Database Server
  billingdbName - name of the Billing database
  webRoot       - The root directory of the web server
  siteDir       - The name of the directory index.cgi is in
  cookieDomain  - The domain that cookies are to be created in
  cookieLife    - The number of minutes that session cookies should stay
  emailAddress  - The email address for JavaScript Error Handler to use
  allowFileUploads - Do we allow file uploads (true/false)
  maxFileUploadSize - Max size of all data coming into the portal
  fileUploadLocation - Location files will be wrote to
  fileUploadPrefix - name prefixed to all incoming files
  hostingURL    - The URL of the hosting companies web site.
  hostingAltTag - The Alt Tag to use for the hostingURL.
  hostingLogoUrl- The URL of the hosting companies logo.
  cookieDebug   - Indicates if the cookie debug info should be shown at
                  the bottom of each portal generated page.

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Base(3)

=cut
