# User.pm - The Object Class that provides the User Form Module
# Created by James A. Pattie, 2004-06-23.

# Copyright (c) 2000-2004 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Forms::User;
use strict;
use Portal::AppState;
use HTMLObject::Form;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::AppState Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.1';

=head1 NAME

User - Object used to build the User Forms Class.

=head1 SYNOPSIS

  use Portal::Forms::User;
  my $obj = Portal::Forms::User->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  
  # ask the $obj to process User related forms.

=head1 DESCRIPTION

User is a User Forms class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Forms::User object.
 See Portal::PortalArgs(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{formObj} = HTMLObject::Form->new();

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

# create the methods to create, edit, view the specified userObj data via the
# forms defined in the Portal::Objects::User object.

=item @ changePassword(user, caller, url, app, state)

 user - specifies the User object of the user whose password is
   to be changed.
 caller - specifies the User object of the person changing the
   password.  This allows us to verify that the user's password
   can be changed (either by themselves, a company admin or a
   system admin).
 url - the url to specify in the forms action.
 app - the app that is changing the users password.
 state - the state in the app that is changing the users password.
   
 returns an array with ($result, \%html) as values, where:
   result: 0 = error occured,
           1 = password successfully changed,
           2 = html needs to be displayed, we are still processing.
   html: hashref that contains the output to be $doc->print(%html)
     into the callers document for displaying to the user.
     
 The resulting form is named: passwordForm
     
=cut
sub changePassword
{
  my $self = shift;
  my %args = ( user => undef, caller => undef, url => "", app => "", state => "", @_ );
  my $user = $args{user};
  my $caller = $args{caller};
  my $url = $args{url};
  my $app = $args{app};
  my $state = $args{state};
  my @result = ( 0, {} );
  my $formName = "";  # keeps track of the userPassword or adminPassword form data to work with.
  
  # make sure I got valid input.
  if (!defined $user)
  {
    $self->missing("user");
  }
  if (!defined $caller)
  {
    $self->missing("caller");
  }
  if (!$user->isValid)
  {
    $self->invalid("user", $user->errorMessage);
  }
  if (!$caller->isValid)
  {
    $self->invalid("caller", $caller->errorMessage);
  }
  if (length $app == 0)
  {
    $self->missing("app");
  }
  if (length $state == 0)
  {
    $self->missing("state");
  }
  if ($user->id != $caller->id)
  {
    # verify the caller is allowed to change the user's password.
    if (!$caller->sysadmin && !$caller->admin)
    {
      $self->invalid("caller", $caller->uname, $self->{langObj}->map("YouAreNotSysOrCompanyAdmin"));
    }
    elsif (!$caller->sysadmin && $caller->admin && ($user->companyId != $caller->companyId))
    {
      $self->invalid("caller", $caller->uname, sprintf($self->{langObj}->map("YouAreCompanyAdminButWrongCompany", $user->{uname})));
    }
    else
    {
      $formName = "adminPassword";
    }
  }
  else
  {
    $formName = "userPassword";
  }
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return @result;
  }
  
  my %form = ();  # holds the result of the $formObj->generate() call.
  my $displayForm = 0;
  if (!exists $self->{input}->{formSubmitted})
  {
    $displayForm = 1;
  }
  else
  {
    # validate the input and update if everything ok.
    my $result = $self->{formObj}->validate(input => $self->{input}, profile => $user->{profile}->{$formName});
    if ($self->{formObj}->didErrorOccur)
    {
      $self->setError($self->{formObj}->errorMessage);
      return @result;
    }
    if (!$result)
    {
      $displayForm = 1;
    }
    else
    {
      # we have a valid form to at least the Data::FormValidator.
      # do any final validation and if everything looks good, update the database.
      
      if ($formName eq "userPassword")
      {
        # make sure the "original" password matches what is the current password.
        if ($user->{password} ne $self->{authObj}->md5EncryptPassword($self->{input}->{original}))
        {
          # remove the original from the valid hash and move it to the invalid hash.
          $self->{formObj}->invalid("original", $self->{formObj}->{'_errors_'}->{valid}->{original});
          delete $self->{formObj}->{'_errors_'}->{valid}->{original};
          $displayForm = 1;
        }
      }
      # make sure the new passwords match.
      if (length $self->{input}->{password1} < 6)
      {
        # remove password1 and password2 from the valid hash and move them to the invalid hash.
        $self->{formObj}->invalid("password1", $self->{formObj}->{'_errors_'}->{valid}->{password1});
        delete $self->{formObj}->{'_errors_'}->{valid}->{password1};
        $self->{formObj}->invalid("password2", $self->{formObj}->{'_errors_'}->{valid}->{password2});
        delete $self->{formObj}->{'_errors_'}->{valid}->{password2};
        $displayForm = 1;
      }
      elsif ($self->{input}->{password1} ne $self->{input}->{password2})
      {
        # remove password2 from the valid hash and move it to the invalid hash.
        $self->{formObj}->invalid("password2", $self->{formObj}->{'_errors_'}->{valid}->{password2});
        delete $self->{formObj}->{'_errors_'}->{valid}->{password2};
        $displayForm = 1;
      }
      if (!$displayForm)
      {
        # at this point, we have a valid password change, so update the users info.
        $user->{password} = $self->{input}->{password1};
        $user->{passCrypted} = 0;
        
        my $result = $self->{authObj}->updateUser(userObj => $user);
        if ($self->{authObj}->didErrorOccur)
        {
          $self->setError($self->{authObj}->errorMessage);
          return @result;
        }
        $result[0] = 1;  # signal we successfully changed the password.
      }
    }
  }
  
  if ($displayForm)
  {
    # display the form.
    my $template = $user->{template}->{$formName};
    
    # remove the #CHILD_FORM_ARGS# and #CHILD_FORM_TABLE# tags, if they haven't been replaced yet.
    $template =~ s/(#CHILD_FORM_(ARGS|TABLE)#)//g;
    
    my $data = $user->{data}->{$formName};
    $data->{app}->{-Value} = $app;
    $data->{state}->{-Value} = $state;
    
    my %form = $self->{formObj}->generate(data => $data,
      template => $template,
      name => "passwordForm",
      action => $url,
      method => "post",
      profile => $user->{profile}->{$formName});
    # update the html hash in the result array.
    if ($self->{formObj}->didErrorOccur)
    {
      $self->setError($self->{formObj}->errorMessage);
      return @result;
    }
    $result[0] = 2;  # signal we have output to display to the user.
    $result[1] = \%form;
  }
  
  return @result;
}

=item % viewUser(user)

 user - specifies the User object of the user to view.
   
 returns %html, where:
   html: hash that contains the output to be $doc->print(%html)
     into the callers document for displaying to the user.
     
 If an error occured, we return undef.
     
=cut
sub viewUser
{
  my $self = shift;
  my %args = ( user => undef, @_ );
  my $user = $args{user};
  
  # make sure I got valid input.
  if (!defined $user)
  {
    $self->missing("user");
  }
  if (!$user->isValid)
  {
    $self->invalid("user", $user->errorMessage);
  }
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return undef;
  }
  
  my %form = ();  # holds the result of the $formObj->generate() call.
  # display the form.
  my $formName = "view";
  my $template = $user->{template}->{$formName};
  
  # remove the #CHILD_FORM_ARGS# and #CHILD_FORM_TABLE# tags, if they haven't been replaced yet.
  $template =~ s/(#CHILD_FORM_(ARGS|TABLE)#)//g;
  
  my $data = $user->{data}->{$formName};
  
  my %form = $self->{formObj}->generate(data => $data,
    template => $template,
    name => "viewUserForm",
    action => "",
    method => "post",
    profile => $user->{profile}->{$formName},
    readOnly => 1, readOnlyMode => "text", readOnlyHidden => 0);
  # update the html hash in the result array.
  if ($self->{formObj}->didErrorOccur)
  {
    $self->setError($self->{formObj}->errorMessage);
    return undef;
  }
  
  return %form;
}

=item @ editUser(user, caller, url, app, state)

 user - specifies the User object of the user who we are editing.
 caller - specifies the User object of the person doing the 
   editing.  This allows us to verify that the user's info
   can be changed (either by themselves, a company admin or a
   system admin).
 url - the url to specify in the forms action.
 app - the app that is changing the users info.
 state - the state in the app that is changing the users info.
   
 returns an array with ($result, \%html, $userObj) as values, where:
   result: 0 = error occured,
           1 = info successfully changed,
           2 = html needs to be displayed, we are still processing.
   html: hashref that contains the output to be $doc->print(%html)
     into the callers document for displaying to the user.
   userObj: the updated userObj when result = 1.
     
 The resulting form is named: userEditForm
     
=cut
sub editUser
{
  my $self = shift;
  my %args = ( user => undef, caller => undef, url => "", app => "", state => "", @_ );
  my $user = $args{user};
  my $caller = $args{caller};
  my $url = $args{url};
  my $app = $args{app};
  my $state = $args{state};
  my @result = ( 0, {} );
  my $formName = "";  # keeps track of the userPassword or adminPassword form data to work with.
  
  # make sure I got valid input.
  if (!defined $user)
  {
    $self->missing("user");
  }
  if (!defined $caller)
  {
    $self->missing("caller");
  }
  if (!$user->isValid)
  {
    $self->invalid("user", $user->errorMessage);
  }
  if (!$caller->isValid)
  {
    $self->invalid("caller", $caller->errorMessage);
  }
  if (length $app == 0)
  {
    $self->missing("app");
  }
  if (length $state == 0)
  {
    $self->missing("state");
  }
  if ($user->id != $caller->id)
  {
    # verify the caller is allowed to change the user's info.
    if (!$caller->sysadmin && !$caller->admin)
    {
      $self->invalid("caller", $caller->uname, $self->{langObj}->map("YouAreNotSysOrCompanyAdmin"));
    }
    elsif (!$caller->sysadmin && $caller->admin && ($user->companyId != $caller->companyId))
    {
      $self->invalid("caller", $caller->uname, sprintf($self->{langObj}->map("YouAreCompanyAdminButWrongCompany", $user->{uname})));
    }
    else
    {
      $formName = "adminEdit";
    }
  }
  else
  {
    $formName = "userEdit";
  }
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return @result;
  }
  
  my %form = ();  # holds the result of the $formObj->generate() call.
  my $displayForm = 0;
  my $errorStr = "";
  if (!exists $self->{input}->{formSubmitted})
  {
    $displayForm = 1;
  }
  else
  {
    # validate the input and update if everything ok.
    my $result = $self->{formObj}->validate(input => $self->{input}, profile => $user->{profile}->{$formName});
    if ($self->{formObj}->didErrorOccur)
    {
      $self->setError($self->{formObj}->errorMessage);
      return @result;
    }
    if (!$result)
    {
      $displayForm = 1;
    }
    else
    {
      # we have a valid form to at least the Data::FormValidator.
      # do any final validation and if everything looks good, update the database.
      
      # build up the values from the $user object and from the user input
      # based upon the mode we are in.
      my %userInfo = ( id => $user->{id}, uname => $user->{uname}, lastEdited => $user->{lastEdited}, 
                       companyId => $user->{companyId}, password => $user->{password} );
      
      if ($formName eq "adminEdit")
      {
        # all other fields are fair game from the user input.
        foreach my $field ( qw( active fname mname lname ssn empId language timeFormat tz admin sysadmin email) )
        {
          $userInfo{$field} = $self->{formObj}->getValidEntry($field);
        }
      }
      else
      {
        # only some of the user input is usable.
        foreach my $field ( qw( fname mname lname language timeFormat tz email ) )
        {
          $userInfo{$field} = $self->{formObj}->getValidEntry($field);
        }
        foreach my $field ( qw( active ssn empId admin sysadmin ) )
        {
          $userInfo{$field} = $user->{$field};
        }
      }
      
      # make sure the new info is valid via the Portal::Objects::User class.
      my $tmpUser = Portal::Objects::User->new(langObj => $self->{langObj});
      $tmpUser->populate(%userInfo);
      if ($tmpUser->didErrorOccur)
      {
        $errorStr = $tmpUser->genErrorString("all");
        # now we have to walk the $tmpUser invalid and missing structures and
        # figure out what entries are causing problems and set them invalid
        # in the formObj structure so we can redisplay the form and have the
        # user try and fix things.
        foreach my $entry ($tmpUser->getInvalid())
        {
          # add to the formObj invalid structure.
          $self->{formObj}->invalid($entry, $tmpUser->getInvalidEntry($entry));
          # clear the formObj valid structure entry.
          delete $self->{formObj}->{'_errors_'}->{valid}->{$entry};
        }
        foreach my $entry ($tmpUser->getMissing())
        {
          # add to the formObj invalid structure.
          $self->{formObj}->missing($entry);
          # clear the formObj valid structure entry.
          delete $self->{formObj}->{'_errors_'}->{valid}->{$entry};
        }
        $displayForm = 1;
      }
      if (!$displayForm)
      {
        # at this point, we have a valid tmpUser, so update the users info.
        
        my $result = $self->{authObj}->updateUser(userObj => $tmpUser);
        if ($self->{authObj}->didErrorOccur)
        {
          $self->setError($self->{authObj}->errorMessage);
          return @result;
        }
        $result[0] = 1;  # signal we successfully updated the user.
        $result[2] = $tmpUser;  # pass back the updated User Object.
      }
    }
  }
  
  if ($displayForm)
  {
    # display the form.
    my $template = $user->{template}->{$formName};
    
    # remove the #CHILD_FORM_ARGS# and #CHILD_FORM_TABLE# tags, if they haven't been replaced yet.
    $template =~ s/(#CHILD_FORM_(ARGS|TABLE)#)//g;
    
    my $data = $user->{data}->{$formName};
    $data->{app}->{-Value} = $app;
    $data->{state}->{-Value} = $state;
    
    my %form = $self->{formObj}->generate(data => $data,
      template => $template,
      name => "editUserForm",
      action => $url,
      method => "post",
      profile => $user->{profile}->{$formName});
    # update the html hash in the result array.
    if ($self->{formObj}->didErrorOccur)
    {
      $self->setError($self->{formObj}->errorMessage);
      return @result;
    }
    if (length $errorStr > 0)
    {
      $form{body} .= "<br />$errorStr\n";
    }
    $result[0] = 2;  # signal we have output to display to the user.
    $result[1] = \%form;
  }
  
  return @result;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::AppState(3)

=cut
