# Help.pm - The Object Class that provides a Help Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2003 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Help;
use strict;
use Portal::Session;
use Portal::SessionHandler;
use Portal::PortalArgs;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::PortalArgs Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.11';

# global variables.
# modify to represent the commands this state provides.
my %commands = ( display => "Display Help Screen", closeHelp => "Close Help Window" );

=head1 NAME

Help - Object used to build a Help Object Class.

=head1 SYNOPSIS

  use Portal::Help;
  my $obj = Portal::Help->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Help is a Help class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(helpApp, content)

 Creates a new instance of the Portal::Help object.
 See Portal::PortalArgs(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{helpApp} = $self->{input}->{helpApp};
  $self->{content} = $self->{input}->{content};
  my @contentArray = split /,/, $self->{content};
  $self->{contentArray} = \@contentArray;

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  # get the portalSession
  $self->{session} = $self->{methods}->portalSession(portalDB => $self->{portalDB},
                                                     cookieObj => $self->{cookieObj},
                                                     appName => "pcx_session_id",
                                                     configObj => $self->{configObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $self->{methods}->errorMessage);
    return $self;
  }
  if ($self->{session}->didErrorOccur)
  {
    $self->setError(errorString => $self->{session}->errorMessage);
    return $self;
  }

  # get the AppSessions
  $self->{appSessions} = $self->{methods}->portalSessionHandler(portalDB => $self->{portalDB},
                                                                sessionObj => $self->{session},
                                                                configObj => $self->{configObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $self->{methods}->errorMessage);
    return $self;
  }
  if ($self->{appSessions}->didErrorOccur)
  {
    $self->setError(errorString => $self->{appSessions}->errorMessage);
    return $self;
  }

  # instantiate the Help Data Structure for the Topics hash.
  my $helpName = ($self->{helpApp} ne "Portal" ? "Portal::" : "") . $self->{helpApp} . "::Data::Help";
  eval "use $helpName;";
  if ($@)
  {
    $self->setError(errorString => "Eval of $helpName failed!<br>\nError = '$@'.<br>\n");
    return $self;
  }
  # retrieve the Help Data object and store it in our object.
  $self->{helpData} = $helpName->new(lang => $self->{langObj}->{lang}, langObj => $self->{langObj});
  if ($self->{helpData}->didErrorOccur)
  {
    $self->setError(errorString => $self->{helpData}->errorMessage);
    return $self;
  }

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (length $self->{content} == 0 && $self->{input}->{command} ne "closeHelp")
  {
    $self->missing("content");
  }
  if (scalar $self->{contentArray} == 0 && $self->{input}->{command} ne "closeHelp")
  {
    $self->setError("No keywords found to work with!<br>\n"); # LANG this.
  }
  if (length $self->{helpApp} == 0)
  {
    $self->missing("helpApp");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject run(state, command)

 Pulls the specified state file into existance and calls
 run(command).

 returns: The HTMLObject document to display or undef if an error.

=cut

sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;

  if (!exists $commands{$command})
  {
    $self->setError(errorString => sprintf($self->langObj->map("commandNotKnown"), $command));
    return undef;
  }

  # store the browserCapabilities hash before the session may go out of scope.
  my %browserCap = %{$self->{session}->{store}->{browserCap}};

  $doc = HTMLObject::Normal->new;
  if ($command eq "display")
  {
    # create/update the entry in the windows hash
    if (!exists $self->{input}->{inlineDocument})
    {
      my $window = "help_" . $self->{helpApp} . "_" . $self->{session}->{store}->{windowTitleTimestamp};
      $self->{session}->{store}->{windows}->{$window} = 1;
      $self->{session}->{store}->{changed} = 1;
    }

    $doc = $self->displayHelp(doc => $doc);
    if ($self->didErrorOccur)
    {
      $self->prefixError();
      return undef;
    }
  }
  elsif ($command eq "closeHelp")
  {
    if (!exists $self->{input}->{inlineDocument})
    {
      # remove the help window from the windows hash
      my $window = "help_" . $self->{helpApp} . "_" . $self->{session}->{store}->{windowTitleTimestamp};
      delete $self->{session}->{store}->{windows}->{$window};
      $self->{session}->{store}->{changed} = 1;

      # generate the JavaScript code to close the window.
      $doc->setTitle("Close Help");
      my $onLoadCode = "window.close();\n";
      $doc->setOnload(code => $onLoadCode);
      $doc->setFocus("body");
      $doc->print("Closing Window");
    }
    else
    {
      $doc->setTitle("Close Help - Invalid");
      $doc->setFocus("body");
      $doc->print("You can not Close the Help Window!<br>\nYou are in an Inline Document!");
    }
  }

  # have it process the session object so that any changes are saved in the session.
  $self->{session}->writeCookie(doc => $doc);

  # set the JavaScript Error Handler Email Address
  $self->{methods}->setJavaScriptErrorInfo(doc => $doc, email => $self->{configObj}->{emailAddress}, appName => "Portal", appVersion => $VERSION);
  if ($self->{methods}->didErrorOccur)
  {
    $self->postfixError($self->{methods}->errorMessage);
    return undef;
  }

  return $doc;
}

sub displayHelp
{
  my $self = shift;
  my %args = ( @_ );
  my $doc = $args{doc};

  my $url = $self->{methods}->createBaseURL(type => "Portal", linkType => "cgi");
  my $closeUrl = $self->{methods}->urlBuilder(doc => $doc, baseUrl => $url, arguments => { app => "Portal", state => "Help", command => "closeHelp", helpApp => $self->{helpApp} });
  my $printUrl = $self->{methods}->urlBuilder(doc => $doc, baseUrl => $url, arguments => { print => 1 });
  my $indexUrl = $self->{methods}->urlBuilder(doc => $doc, baseUrl => $url, arguments => { app => "Portal", state => "Help", command => "display", helpApp => $self->{helpApp}, content => $self->{contentArray}[0] });
  my $helpUrl = $self->{methods}->urlBuilder(doc => $doc, baseUrl => $url, arguments => { app => "Portal", state => "Help", command => "display" }) . "&helpApp=\%s&content=\%s";

  # get the users colorScheme to work with.
  my $colorScheme = $self->{applicationObj}->getCurrentColorScheme(userId => $self->{session}->{store}->{userObj}->{id});
  if ($self->{applicationObj}->didErrorOccur)
  {
    $self->setError(errorString => $self->{applicationObj}->errorMessage);
    return undef;
  }
  if (!defined $colorScheme)
  {
    $self->setError(errorString => "No valid colorScheme returned!<br>\n");
    return undef;
  }

  my $closeWindow = $self->langObj->map("closeWindow");
  my $help = $self->langObj->map("help");
  my $print = $self->langObj->map("print");

  $doc->setTitle(sprintf("%s - Help Screen", $self->{helpApp}));
  # create the needed css entries
  $doc = $self->{methods}->generateColorSchemeCSS(doc => $doc, colorScheme => $colorScheme);
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $self->{methods}->errorMessage);
    return undef;
  }

  # create the showHelp JS function.
  $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "help", langObj => $self->{langObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $self->{methods}->errorMessage);
    return undef;
  }

  # build up the menuItems arrays
  my @itemsLeft = ();
  my @itemsRight = ();

  if (exists $self->{input}->{inlineDocument})
  {
    $itemsLeft[0] = Portal::Objects::MenuItem->new(type => "text", text => "$self->{helpApp}", langObj => $self->{langObj});
  }
  else
  {
    $itemsLeft[0] = Portal::Objects::MenuItem->new(type => "link", url => "$closeUrl",
                    text => $closeWindow, title => $closeWindow, onMouseOver => "window.status='$closeWindow'; return true;", langObj => $self->{langObj});
  }
  $itemsRight[0] = Portal::Objects::MenuItem->new(type => "link", url => "$printUrl", onClick => "window.print(); return false;",
                   text => $print, title => $print, onMouseOver => "window.status='$print'; return true;", langObj => $self->{langObj});

  # now validate they created ok.
  for (my $i=0; $i < scalar @itemsLeft; $i++)
  {
    if ($itemsLeft[$i]->didErrorOccur)
    {
      $self->setError(errorString => $itemsLeft[$i]->errorMessage);
      return undef;
    }
  }
  for (my $i=0; $i < scalar @itemsRight; $i++)
  {
    if ($itemsRight[$i]->didErrorOccur)
    {
      $self->setError(errorString => $itemsRight[$i]->errorMessage);
      return undef;
    }
  }
  my $menuString = $self->{methods}->displayMenu(doc => $doc, orientation => "horizontal",
                  itemsLeft => \@itemsLeft, itemsRight => \@itemsRight,
                  indent => 0);
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $self->{methods}->errorMessage);
    return undef;
  }

  $doc->setFocus("body");
  $doc->print($menuString);
  $doc->print(<<"END_OF_CODE");
<hr width="100%">
END_OF_CODE

  # now display the list of topics that got us to this point.
  my $counter;
  for ($counter=0; $counter < scalar @{$self->{contentArray}}; $counter++)
  {
    my $url = sprintf($helpUrl, $doc->encodeString(string => $self->{helpApp}), $doc->encodeString(string => join ',', @{$self->{contentArray}}[0..$counter]));
    $url .= "&inlineDocument=1" if (exists $self->{input}->{inlineDocument});
    $doc->print("\n<br>\n") if ($counter > 0);
    $doc->print("&nbsp;&nbsp;&nbsp;&nbsp;" x $counter);  # seperator token
    $doc->print("<a href=\"$url\" class=\"help\" onMouseOver=\"window.status='$self->{contentArray}[$counter]'; return true;\" title=\"$self->{contentArray}[$counter]\">$self->{contentArray}[$counter]</a>");
  }
  $doc->print("\n<br>\n<hr width=\"100%\">\n<br>\n");  # drop display down 2 lines.

  # now display the topics or info for this item.
  my $topicRef = $self->{helpData}->{Topics}->{$self->{helpData}->{lang}};  # This points us to the language hash.
  for ($counter=0; $counter < scalar @{$self->{contentArray}}; $counter++)
  {
    if (! exists $topicRef->{$self->{contentArray}[$counter]})
    {
      $self->setError(errorString => "Topic = '$self->{contentArray}[$counter]' does not exist!<br>\n");
      return undef;
    }
    $topicRef = $topicRef->{$self->{contentArray}[$counter]};
    if ($counter < scalar @{$self->{contentArray}} - 1 && ref($topicRef) ne "HASH")
    { # Problem!!!
      $self->setError(errorString => "Topic = '$self->{contentArray}[$counter]' is not a HASH and we are not drilled all the way down yet! &nbsp;It is a " . ref($topicRef) . "!<br>\n");
      return undef;
    }
  }

  # we are now drilled down to the point the user specified, now display either the list of available topics (hash) or the info for the specified topic.
  if (ref $topicRef eq "HASH")
  {
    # get the _order_ entry and build the array of items to display in the user specified order.
    my $order = $topicRef->{_order_};
    if (length $order == 0)
    {
      $self->setError(errorString => "_order_ not specified for Topic = '$self->{content}'!<br>\n");
      return undef;
    }

    if (exists $topicRef->{_overview_})
    {
      $doc->print(<<"END_OF_CODE");
<table border="0" cellpadding="0" cellspacing="0" width="100%">
  <tr class="helpTopic">
    <td align="left" width="100%">$topicRef->{_overview_}</td>
  </tr>
  <tr>
    <td>&nbsp;</td>
  </tr>
</table>
END_OF_CODE
    }

    my @order = split /,/, $order;
    # start a table.
    $doc->print(<<"END_OF_CODE");
<table border="0" cellpadding="0" cellspacing="0" width="50%">
END_OF_CODE
    foreach my $topic (@order)
    {
      if (! exists $topicRef->{$topic} && $topic !~ /^(_seperator_|_line_)$/)
      {
        $self->setError(errorString => "Topic Item = '$topic' does not exist!<br>\n");
        return undef;
      }

      my @tmpArray = @{$self->{contentArray}};
      $tmpArray[++$#tmpArray] = $topic;
      my $url = sprintf($helpUrl, $doc->encodeString(string => $self->{helpApp}), $doc->encodeString(string => join ',', @tmpArray));
      $url .= "&inlineDocument=1" if (exists $self->{input}->{inlineDocument});

      # figure out what needs to be displayed.  A link, seperator, line, etc.
      my $content = "<a href=\"$url\" class=\"help\" onMouseOver=\"window.status='$topic'; return true;\" title=\"$topic\">$topic</a>";
      $content = "<br>" if ($topic eq "_seperator_");
      $content = "<hr width=\"25%\" align=\"left\">" if ($topic eq "_line_");

      $doc->print(<<"END_OF_CODE");
  <tr>
    <td align="left" width="10%">&nbsp;</td>
    <td align="left" width="90%">$content</td>
  </tr>
END_OF_CODE
    }
    $doc->print(<<"END_OF_CODE");
</table>
END_OF_CODE
  }
  else # info
  {
    # we need to do substitutions for <help></help> tags in the string.
    if ($topicRef =~ /<help app=".+" content=".+">.+<\/help>/)
    {
      $topicRef =~ s/(<help app=".+" content=".+">.+<\/help>)/MYHELPTOKEN/;
      my $helpString = $1;
      my @tokens = split / /, $helpString;  # tokens I want are 1 and 2. 2 needs more work though.
      my $app = $tokens[1];
      $app =~ s/app="(.+)"/$1/;
      my $content = $tokens[2];
      @tokens = split />/, $content;  # tokens I want are 0 and 1

      $content = $tokens[0];
      $content =~ s/content="(.+)"/$1/;
      # fixup the _ to space conversions or \_ to _ conversions.
      $content =~ s/([^\\])(_)/$1 /g;
      $content =~ s/\\_/_/g;
      $content = $doc->encodeString(string => $content);  # make sure it is url encoded.

      my $name = $tokens[1];
      $name =~ s/(.+)<\/help/$1/;

      # do _ replacement on $name but only on those that are not \_ escaped.
      $name =~ s/([^\\])(_)/$1 /g;
      # now replace any \_ with just _.
      $name =~ s/\\_/_/g;

      # generate the url string and replace for MYHELPTOKEN.
      my $url = sprintf($helpUrl, $app, $content);
      $url .= "&inlineDocument=1" if (exists $self->{input}->{inlineDocument} && $app eq $self->{helpApp});
      my $window = "help_" . $app . "_" . $self->{session}->{store}->{windowTitleTimestamp};
      my $urlString = "<a href=\"" . ($app eq $self->{helpApp} ? $url : "#") . "\" class=\"help\" onMouseOver=\"window.status='$name'; return true;\" title =\"$name\"" . ($app eq $self->{helpApp} ? "" : " onClick=\"showHelp('$window', '$url', 480, 640); return false;\"") . ">$name</a>";

      $topicRef =~ s/MYHELPTOKEN/$urlString/;
    }
    $doc->print(<<"END_OF_CODE");
<table border="0" cellpadding="0" cellspacing="0" width="100%">
  <tr class="helpTopic">
    <td align="left" width="100%">$topicRef</td>
  </tr>
  <tr>
    <td>&nbsp;</td>
  </tr>
</table>
END_OF_CODE
  }

  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::PortalArgs(3)

=cut
