# CompanyApplicationObject.pm - The Object Class that provides a CompanyApplicationObject Object
# Created by James A. Pattie, 11/07/2000.

# Copyright (c) 2000-2003 Xperience, Inc. http://www.pcxperience.com/
# All rights reserved.  This program is free software; you can redistribute it
# and/or modify it under the same terms as Perl itself.

package Portal::Objects::CompanyApplicationObject;
use strict;
use Portal::Base;
use Portal::Data::Variables;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.07';

=head1 NAME

CompanyApplicationObject - Object used to build a CompanyApplicationObject Object Class.

=head1 SYNOPSIS

  use Portal::Objects::CompanyApplicationObject;
  my $obj = Portal::Objects::CompanyApplicationObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

CompanyApplicationObject is a CompanyApplicationObject class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(appId, companyId, number, server, port, cost, unit, dbName,
dbHost, dbType, height, width, autorun, wap)

 Creates a new instance of the Portal::Objects::CompanyApplicationObject module.
 See Portal::Base(3) for a listing of required arguments.

 requires:
   appId
   companyId
   number
   server
   port
   cost
   unit
   dbHost
   dbName
   dbType
   height
   width
   autorun
   wap

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( appId => -1, companyId => -1, number => 1, server => "", port => -1, cost => "0.00", unit => 0, dbName => "", height => "640",
               width => "480", autorun => 0, dbType => "Pg", dbPort => "5432", wap => 0, dbHost => "127.0.0.1", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{appId} = $args{appId};
  $self->{companyId} = $args{companyId};
  $self->{number} = $args{number};
  $self->{server} = $args{server};
  $self->{port} = $args{port};
  $self->{cost} = $args{cost};
  $self->{unit} = $args{unit};
  $self->{dbName} = $args{dbName};
  $self->{height} = $args{height};
  $self->{width} = $args{width};
  $self->{autorun} = $args{autorun};
  $self->{dbHost} = $args{dbHost};
  $self->{dbType} = $args{dbType};
  $self->{dbPort} = $args{dbPort};
  $self->{wap} = $args{wap};
  $self->{variables} = Portal::Data::Variables->new(langObj => $self->{langObj});

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->{appId} < 0)
  {
    $self->invalid("appId", "$self->{appId}");
  }
  if ($self->{companyId} < 0)
  {
    $self->invalid("companyId", "$self->{companyId}");
  }
  if ($self->{number} < 0)  # 0 is valid as it represents unlimited licenses.
  {
    $self->invalid("number", "$self->{number}");
  }
  if (length $self->{server} == 0)
  {
    $self->invalid("server", "$self->{server}");
  }
  if ($self->{port} < 0)
  {
    $self->invalid("port", "$self->{port}");
  }
  if ($self->{cost} !~ /^\d+\.\d{2}$/)
  {
    $self->invalid("cost", "$self->{cost}");
  }
  if (!exists $self->{variables}->{units}->{$self->{unit}})
  {
    $self->invalid("unit", "$self->{unit}");
  }
  if (length $self->{dbName} == 0)
  {
    $self->invalid("dbName", "$self->{dbName}");
  }
  if ($self->{height} < 0 || $self->{height} > 4000)  # I think 4000 is way too much!
  {
    $self->invalid("height", "$self->{height}");
  }
  if ($self->{width} < 0 || $self->{width} > 4000)
  {
    $self->invalid("width", "$self->{width}");
  }
  if ($self->{autorun} !~ /^(1|0)$/)
  {
    $self->invalid("autorun", "$self->{autorun}");
  }
  if (length $self->{dbHost} == 0)
  {
    $self->missing("dbHost");
  }
  if ($self->{dbType} !~ /^(Pg|mysql)$/)
  {
    $self->invalid("dbType", "$self->{dbType}");
  }
  if ($self->{dbPort} !~ /^\d+$/)
  {
    $self->invalid("dbPort", "$self->{dbPort}");
  }
  if ($self->{wap} !~ /^(1|0)$/)
  {
    $self->invalid("wap", "$self->{wap}");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 Exported VARIABLES

  appId     - int
  companyId - int
  number    - int (number of licenses bought)
  server    - text
  port      - numeric(6)
  dbHost    - text
  dbType    - text
  dbPort    - numeric(6)
  cost      - money (without $ and ,'s)
  unit      - int
  dbName    - text
  height    - int
  width     - int
  autorun   - bool
  wap       - bool


=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Application(3), Portal::Base(3)

=cut
